#!/usr/bin/php -q
<?php

#
# cacti script to read PowerAlert battery charge and load stats with snmp
#

include(dirname(__FILE__) . "/../include/global.php");
include(dirname(__FILE__) . "/../lib/snmp.php");

#
# global declarations
#
global $snmp_hostname;
global $snmp_community;
global $snmp_version;
global $snmp_username;
global $snmp_password;
global $snmp_port;
global $snmp_timeout;
global $snmp_array;
global $cacti_request;
global $cacti_request_type;
global $cacti_request_index;
global $oid;
global $cmdtool;
global $cmdoutput;
global $cmdoutputcount;
global $outputcount;
global $devicecount;

#
# make sure enough parameters were provided
#
if (count($_SERVER['argv']) < 5) {

	echo ("failed: insufficient parameters provided\n");

	displaySyntax();

	exit(1);
}

#
# 1st parameter is always $snmp_hostname
#
$snmp_hostname = substr($_SERVER['argv'][1],0,(strlen($_SERVER['argv'][1]) - 1));

if ($snmp_hostname == "") {

	echo ("failed: no target hostname was specified\n");

	displaySyntax();

	exit(1);
}

#
# 2nd parameter is always $snmp_community
#
$snmp_community = substr($_SERVER['argv'][2],0,(strlen($_SERVER['argv'][2]) - 1));

if ($snmp_community == "") {

	echo ("failed: no SNMP community string was specified\n");

	displaySyntax();

	exit(1);
}

#
# 3rd parameter is always $snmp_version
#
$snmp_version = substr($_SERVER['argv'][3],0,(strlen($_SERVER['argv'][3]) - 1));

if (($snmp_version != 1) and ($snmp_version != 2)) {

	echo ("failed: \"$snmp_version\" is not a valid SNMP version\n");

	displaySyntax();

	exit(1);
}

#
# 4th parameter is always $snmp_port
#
$snmp_port = substr($_SERVER['argv'][4],0,(strlen($_SERVER['argv'][4]) - 1));

if (is_numeric($snmp_port) == FALSE) {

	echo ("failed: \"$snmp_port\" is not a numeric port number \n");

	displaySyntax();

	exit(1);
}

#
# 5th parameter is always $snmp_timeout
#
$snmp_timeout = substr($_SERVER['argv'][5],0,(strlen($_SERVER['argv'][5]) - 1));

if (is_numeric($snmp_timeout) == FALSE) {

	echo ("failed: \"$snmp_timeout\" is not a numeric timeout value \n");

	displaySyntax();

	exit(1);
}

#
# Cacti's SNMP timeout uses milliseconds, while PHP uses the value
# from Net-SNMP's header files, which is typically microseconds.
# Normalize by multiplying the timeout parameter value by 1000.
#
$snmp_timeout = ($snmp_timeout * 1000);

#
# 6th parameter is always $cacti_request
#
$cacti_request = strtolower($_SERVER['argv'][6]);

if (($cacti_request == "query") ||
	($cacti_request == "get")) {

	if (isset($_SERVER['argv'][7]) === FALSE) {

		echo ("failed: incomplete cacti request\n");

		displaySyntax();

		exit(1);
	}

	else {
		$cacti_request_type = strtolower($_SERVER['argv'][7]);
	}

	if ($cacti_request == "get") {

		if (isset($_SERVER['argv'][8]) === FALSE) {

			echo ("failed: incomplete cacti request\n");

			displaySyntax();

			exit(1);
		}

		else {

			$cacti_request_index = $_SERVER['argv'][8];

			if (($cacti_request_index == "") ||
				($cacti_request_index < 0)) {

				echo ("failed: unknown cacti index value\n");

				displaySyntax();

				exit(1);
			}
		}
	}
}

elseif ($cacti_request !== "index") {

	echo ("failed: unknown cacti request\n");

	displaySyntax();

	exit(1);
}

#
# define the root OID where we will look for entries
#
$oid = array (
	array ("upscount", ".1.3.6.1.4.1.850.10.2.1.0"),
	array ("labels", ".1.3.6.1.4.1.850.10.2.3.1.1.4."),
	array ("values", ".1.3.6.1.4.1.850.10.2.3.1.1.6."));

#
# discover each UPS and their index key
#
$devicecount = cacti_snmp_get($snmp_hostname,
	$snmp_community,
	$oid[0][1],
	$snmp_version,
	$snmp_username,
	$snmp_password,
	$snmp_port,
	$snmp_timeout);

if (($devicecount == "") || ($devicecount == 0)) {

	echo ("failed: no UPS devices were discovered");

	exit(1);
}

$cmdoutputcount = 0;

while ($cmdoutputcount < $devicecount) {

	$cmdoutput[$cmdoutputcount] = array (0 => ($cmdoutputcount + 1));

	$cmdoutputcount++;
}

#
# queries require additional processing for values
#
if (($cacti_request == "query") || ($cacti_request == "get")) {

	#
	# for each UPS, fill in the cmdoutput array
	#
	$cmdoutputcount = 0;

	while ($cmdoutputcount < $devicecount) {

		#
		# walk the device-specific "labels" tree
		#
		$scratch_array = cacti_snmp_walk($snmp_hostname,
			$snmp_community,
			($oid[1][1] . ($cmdoutputcount + 1)),
			$snmp_version,
			$snmp_username,
			$snmp_password,
			$snmp_port,
			$snmp_timeout);

		foreach ($scratch_array as $scratch_value) {

			if (($cacti_request_type == "upsname") &&
				($scratch_value['value'] == "Device Name")) {

				preg_match('/(\d+)$/', $scratch_value['oid'], $scratch);

				$scratch_oid = ($oid[2][1] . ($cmdoutputcount + 1) . "." . $scratch[1]);

				$cmdoutput[$cmdoutputcount][1] = cacti_snmp_get($snmp_hostname,
					$snmp_community,
					$scratch_oid,
					$snmp_version,
					$snmp_username,
					$snmp_password,
					$snmp_port,
					$snmp_timeout);

				if ($cmdoutput[$cmdoutputcount][1] == "") {

					$cmdoutput[$cmdoutputcount][1] = "Unnamed UPS";
				}
			}

			if (($cacti_request_type == "outputload") &&
				($scratch_value['value'] == "Output Load")) {

				preg_match('/(\d+)$/', $scratch_value['oid'], $scratch);

				$scratch_oid = ($oid[2][1] . ($cmdoutputcount + 1) . "." . $scratch[1]);

				$cmdoutput[$cmdoutputcount][2] = cacti_snmp_get($snmp_hostname,
					$snmp_community,
					$scratch_oid,
					$snmp_version,
					$snmp_username,
					$snmp_password,
					$snmp_port,
					$snmp_timeout);

				if ($cmdoutput[$cmdoutputcount][2] == "") {

					$cmdoutput[$cmdoutputcount][2] = 0;
				}
			}

			if (($cacti_request_type == "batterycharge") &&
				($scratch_value['value'] == "Battery Charge Remaining")) {

				preg_match('/(\d+)$/', $scratch_value['oid'], $scratch);

				$scratch_oid = ($oid[2][1] . ($cmdoutputcount + 1) . "." . $scratch[1]);

				$cmdoutput[$cmdoutputcount][3] = cacti_snmp_get($snmp_hostname,
					$snmp_community,
					$scratch_oid,
					$snmp_version,
					$snmp_username,
					$snmp_password,
					$snmp_port,
					$snmp_timeout);

				if ($cmdoutput[$cmdoutputcount][3] == "") {

					$cmdoutput[$cmdoutputcount][3] = 0;
				}
			}

			if (($cacti_request_type == "inputvoltage") &&
				($scratch_value['value'] == "Input Voltage")) {

				preg_match('/(\d+)$/', $scratch_value['oid'], $scratch);

				$scratch_oid = ($oid[2][1] . ($cmdoutputcount + 1) . "." . $scratch[1]);

				$cmdoutput[$cmdoutputcount][4] = cacti_snmp_get($snmp_hostname,
					$snmp_community,
					$scratch_oid,
					$snmp_version,
					$snmp_username,
					$snmp_password,
					$snmp_port,
					$snmp_timeout);

				if ($cmdoutput[$cmdoutputcount][4] == "") {

					$cmdoutput[$cmdoutputcount][4] = 0;
				}

				if ($cmdoutput[$cmdoutputcount][4] < 50) {

					$cmdoutput[$cmdoutputcount][4] =
						$cmdoutput[$cmdoutputcount][4] * 10;
				}
			}

			if (($cacti_request_type == "inputfrequency") &&
				(($scratch_value['value'] == "Input Frequency") ||
				($scratch_value['value'] == "Frequency"))) {

				preg_match('/(\d+)$/', $scratch_value['oid'], $scratch);

				$scratch_oid = ($oid[2][1] . ($cmdoutputcount + 1) . "." . $scratch[1]);

				$cmdoutput[$cmdoutputcount][5] = cacti_snmp_get($snmp_hostname,
					$snmp_community,
					$scratch_oid,
					$snmp_version,
					$snmp_username,
					$snmp_password,
					$snmp_port,
					$snmp_timeout);

				if ($cmdoutput[$cmdoutputcount][5] == "") {

					$cmdoutput[$cmdoutputcount][5] = 0;
				}
			}

			if (($cacti_request_type == "batteryvoltage") &&
				($scratch_value['value'] == "Battery Voltage")) {

				preg_match('/(\d+)$/', $scratch_value['oid'], $scratch);

				$scratch_oid = ($oid[2][1] . ($cmdoutputcount + 1) . "." . $scratch[1]);

				$cmdoutput[$cmdoutputcount][6] = cacti_snmp_get($snmp_hostname,
					$snmp_community,
					$scratch_oid,
					$snmp_version,
					$snmp_username,
					$snmp_password,
					$snmp_port,
					$snmp_timeout);

				if ($cmdoutput[$cmdoutputcount][6] == "") {

					$cmdoutput[$cmdoutputcount][6] = 0;
				}
			}
		}

		$cmdoutputcount++;
	}
}

#
# generate output
#
$outputcount = 0;

while ($outputcount < count($cmdoutput)) {

	switch ($cacti_request) {

	#
	# if they want an index, dump a list of numbers
	#
	case "index":

		echo ($cmdoutput[($outputcount)][0] . "\n");

		break;

	#
	# if they want query data, give them the type they want
	#
	case "query":

		switch ($cacti_request_type) {

		#
		# they want the index numbers
		#
		case "upsdevice":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][0] . "\n");

			break;

		#
		# they want the UPS name
		#
		case "upsname":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][1] . "\n");

			break;

		#
		# they want the output load readings
		#
		case "outputload":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][2] . "\n");

			break;

		#
		# they want the battery charge remaining readings
		#
		case "batterycharge":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][3] . "\n");

			break;

		#
		# they want the input voltage readings
		#
		case "inputvoltage":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][4] . "\n");

			break;

		#
		# they want the input frequency readings
		#
		case "inputfrequency":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][5] . "\n");

			break;

		#
		# they want the battery voltage readings
		#
		case "batteryvoltage":

			echo ($cmdoutput[($outputcount)][0] . ":" .
				$cmdoutput[($outputcount)][6] . "\n");

			break;
		}

		break;

	#
	# if they want explicit data, give them the type they want
	#
	case "get":

		if ($cacti_request_index == ($outputcount + 1)) {

			switch ($cacti_request_type) {

			#
			# they want the index numbers
			#
			case "upsdevice":
	
				echo ($cmdoutput[($outputcount)][0] . "\n");
	
				break;
	
			#
			# they want the UPS name
			#
			case "upsname":
	
				echo ($cmdoutput[($outputcount)][1] . "\n");
	
				break;
	
			#
			# they want the output load readings
			#
			case "outputload":
	
				echo ($cmdoutput[($outputcount)][2] . "\n");
	
				break;
	
			#
			# they want the battery charge remaining readings
			#
			case "batterycharge":
	
				echo ($cmdoutput[($outputcount)][3] . "\n");
	
				break;
	
			#
			# they want the input voltage readings
			#
			case "inputvoltage":
	
				echo ($cmdoutput[($outputcount)][4] . "\n");
	
				break;
	
			#
			# they want the input frequency readings
			#
			case "inputfrequency":
	
				echo ($cmdoutput[($outputcount)][5] . "\n");
	
				break;
	
			#
			# they want the battery voltage readings
			#
			case "batteryvoltage":
	
				echo ($cmdoutput[($outputcount)][6] . "\n");
	
				break;
			}

			break;
		}
	}

	$outputcount++;
}

exit(0);

#
# find all array entries with a "value" key whose value is "Device Name"
#
function deviceName($scratch) {

	if ($scratch['value'] == "Device Name") {

		return(true);
	}

	else {
		return(false);
	}
}

function displaySyntax() {

	#
	# display the syntax
	#
	echo ("syntax: " . $_SERVER['argv'][0] . " <hostname>, [<snmp_community>], <snmp_version>, \n" .
		"   <snmp_port>, <snmp_timeout> (index|query <fieldname>|get <fieldname> <index>)\n");
}

?>
